@@ -2,21 +2,14 @@ from nose.tools import *
 from mock import MagicMock
 from . import TestCase, helpers as h
 
-from flask import g
-
 from annotator import es
 from annotator.annotation import Annotation
 
 class TestAnnotation(TestCase):
     def setup(self):
         super(TestAnnotation, self).setup()
-        self.ctx = self.app.test_request_context(path='/api')
-        self.ctx.push()
-
-        g.user = None
 
     def teardown(self):
-        self.ctx.pop()
         super(TestAnnotation, self).teardown()
 
     def test_new(self):
@@ -34,19 +27,21 @@ class TestAnnotation(TestCase):
         a.es = MagicMock()
         a.es.index = 'foo'
         a.save()
-        a.es.conn._send_request.assert_called_with('POST', '/foo/_refresh')
+        args, kwargs = a.es.conn.index.call_args
+        assert_equal(kwargs['refresh'], True)
 
     def test_save_refresh_disable(self):
         a = Annotation(name='bob')
         a.es = MagicMock()
         a.es.index = 'foo'
         a.save(refresh=False)
-        a.es.conn._send_request.assert_not_called_with('POST', '/foo/_refresh')
+        args, kwargs = a.es.conn.index.call_args
+        assert_equal(kwargs['refresh'], False)
 
     def test_fetch(self):
         a = Annotation(foo='bar')
         a.save()
-        b = Annotation.fetch(a.id)
+        b = Annotation.fetch(a['id'])
         assert_equal(b['foo'], 'bar')
 
     def test_delete(self):
@@ -57,14 +52,14 @@ class TestAnnotation(TestCase):
         newann.delete()
 
         noann = Annotation.fetch(1)
-        assert noann == None
+        assert_true(noann == None)
 
     def test_basics(self):
         user = "alice"
         ann = Annotation(text="Hello there", user=user)
         ann['ranges'] = []
-        ann['ranges'].append({})
-        ann['ranges'].append({})
+        ann['ranges'].append({'startOffset': 3})
+        ann['ranges'].append({'startOffset': 5})
         ann['document'] = {
             'title': 'Annotation for Dummies',
             'link': [
@@ -73,7 +68,7 @@ class TestAnnotation(TestCase):
         }
         ann.save()
 
-        ann = Annotation.fetch(ann.id)
+        ann = Annotation.fetch(ann['id'])
         assert_equal(ann['text'], "Hello there")
         assert_equal(ann['user'], "alice")
         assert_equal(len(ann['ranges']), 2)
@@ -108,22 +103,22 @@ class TestAnnotation(TestCase):
         res = Annotation.count(limit=1)
         assert_equal(res, 3)
 
-        res = Annotation.search(uri=uri1)
+        res = Annotation.search(query={'uri':uri1})
         assert_equal(len(res), 2)
         assert_equal(res[0]['uri'], uri1)
-        assert_equal(res[0]['id'], anno2.id)
+        assert_equal(res[0]['id'], anno2['id'])
 
-        res = Annotation.search(user=user1)
+        res = Annotation.search(query={'user':user1})
         assert_equal(len(res), 2)
         assert_equal(res[0]['user'], user1)
-        assert_equal(res[0]['id'], anno3.id)
+        assert_equal(res[0]['id'], anno3['id'])
 
-        res = Annotation.search(user=user1, uri=uri2)
+        res = Annotation.search(query={'user':user1, 'uri':uri2})
         assert_equal(len(res), 1)
         assert_equal(res[0]['user'], user1)
-        assert_equal(res[0]['id'], anno3.id)
+        assert_equal(res[0]['id'], anno3['id'])
 
-        res = Annotation.count(user=user1, uri=uri2)
+        res = Annotation.count(query={'user':user1, 'uri':uri2})
         assert_equal(res, 1)
 
     def test_search_permissions_null(self):
@@ -133,8 +128,8 @@ class TestAnnotation(TestCase):
         res = Annotation.search()
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('bob')
-        res = Annotation.search()
+        user = h.MockUser('bob')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 0)
 
     def test_search_permissions_simple(self):
@@ -146,16 +141,16 @@ class TestAnnotation(TestCase):
         res = Annotation.search()
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('alice', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('alice', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('bob')
-        res = Annotation.search()
+        user = h.MockUser('bob')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('bob', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('bob', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
     def test_search_permissions_world(self):
@@ -167,16 +162,16 @@ class TestAnnotation(TestCase):
         res = Annotation.search()
         assert_equal(len(res), 1)
 
-        g.user = h.MockUser('alice', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('alice', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
-        g.user = h.MockUser('bob')
-        res = Annotation.search()
+        user = h.MockUser('bob')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
-        g.user = h.MockUser('bob', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('bob', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
     def test_search_permissions_authenticated(self):
@@ -188,12 +183,12 @@ class TestAnnotation(TestCase):
         res = Annotation.search()
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('alice', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('alice', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
-        g.user = h.MockUser('bob', 'anotherconsumer')
-        res = Annotation.search()
+        user = h.MockUser('bob', 'anotherconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
 
@@ -207,12 +202,12 @@ class TestAnnotation(TestCase):
         res = Annotation.search()
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('bob', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('bob', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
-        g.user = h.MockUser('alice', 'anotherconsumer')
-        res = Annotation.search()
+        user = h.MockUser('alice', 'anotherconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 0)
 
     def test_search_permissions_owner(self):
@@ -224,8 +219,8 @@ class TestAnnotation(TestCase):
         res = Annotation.search()
         assert_equal(len(res), 0)
 
-        g.user = h.MockUser('alice', 'testconsumer')
-        res = Annotation.search()
+        user = h.MockUser('alice', 'testconsumer')
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
     def test_search_permissions_malicious(self):
@@ -236,9 +231,9 @@ class TestAnnotation(TestCase):
         anno.save()
 
         # Any user whose username starts with "group:" must be refused any results
-        g.user = h.MockUser('group:anyone', 'testconsumer')
-        res = Annotation.search()
-        assert_equal(len(res), 0)
+        user = h.MockUser('group:anyone', 'testconsumer')
+        search_action = lambda: Annotation.search(user=user)
+        assert_raises(RuntimeError, search_action)
 
     def test_search_permissions_admin(self):
         anno = Annotation(text='Foobar',
@@ -246,18 +241,59 @@ class TestAnnotation(TestCase):
                           consumer='testconsumer')
         anno.save()
 
-        g.user = h.MockUser('bob', 'testconsumer')
-        g.user.is_admin = True
+        user = h.MockUser('bob', 'testconsumer')
+        user.is_admin = True
 
-        res = Annotation.search()
+        res = Annotation.search(user=user)
         assert_equal(len(res), 1)
 
+    def test_search_raw(self):
+        perms = {'read': ['group:__world__']}
+        uri1 = u'http://xyz.com'
+        uri2 = u'urn:uuid:xxxxx'
+        user1 = u'levin'
+        user2 = u'anna'
+        anno1 = Annotation(uri=uri1, text=uri1, user=user1, permissions=perms)
+        anno2 = Annotation(uri=uri1, text=uri1 + uri1, user=user2, permissions=perms)
+        anno3 = Annotation(uri=uri2, text=uri2, user=user1, permissions=perms)
+        anno1.save()
+        anno2.save()
+        anno3.save()
+
+        hits = Annotation.search_raw()
+        assert_equal(len(hits), 3)
+
+        query = {
+            'query': {
+                'filtered': {
+                    'filter': {
+                        'term': {
+                            'user': user1
+                        }
+                    }
+                }
+            }
+        }
+        params = {
+            'from_': 1
+        }
+
+        hits = Annotation.search_raw(query=query)
+        assert_equal(len(hits), 2)
+
+        hits = Annotation.search_raw(params=params)
+        assert_equal(len(hits), 2)
+
+        hits = Annotation.search_raw(query=query, params=params)
+        assert_equal(len(hits), 1)
+
+
     def test_cross_representations(self):
 
-        # create an annotation for an html document which we can 
+        # create an annotation for an html document which we can
         # scrape some document metadata from, including a link to a pdf
 
-        a1 = Annotation(uri='http://example.com/1234', 
+        a1 = Annotation(uri='http://example.com/1234',
                         text='annotation1',
                         user='alice',
                         document = {
@@ -278,7 +314,7 @@ class TestAnnotation(TestCase):
         # create an annotation for the pdf that lacks document metadata since
         # annotator doesn't currently extract information from pdfs
 
-        a2 = Annotation(uri='http://example.com/1234.pdf', 
+        a2 = Annotation(uri='http://example.com/1234.pdf',
                         text='annotation2',
                         user='alice',
                         consumer='testconsumer')
@@ -286,10 +322,33 @@ class TestAnnotation(TestCase):
 
         # now a query for annotations of the pdf should yield both annotations
 
-        g.user = h.MockUser('alice', 'testconsumer')
-        res = Annotation.search(uri='http://example.com/1234.pdf')
+        user = h.MockUser('alice', 'testconsumer')
+        res = Annotation.search(user=user,
+                                query={'uri':'http://example.com/1234.pdf'})
         assert_equal(len(res), 2)
-        
+
         # and likewise for annotations of the html
-        res = Annotation.search(uri='http://example.com/1234')
+        res = Annotation.search(user=user,
+                                query={'uri':'http://example.com/1234'})
         assert_equal(len(res), 2)
+
+    def test_case_sensitivity(self):
+        """Indexing and search should not apply lowercase to strings
+           (this requirement might be changed sometime)
+        """
+        # https://github.com/openannotation/annotator-store/issues/73
+        anno = Annotation(uri='http://example.com/1234',
+                          text='Foobar',
+                          user='alice',
+                          consumer='testconsumer',
+                          custom_field='CaseSensitive')
+        anno.save()
+
+        user = h.MockUser('alice', 'testconsumer')
+        res = Annotation.search(user=user,
+                                query={'custom_field':'CaseSensitive'})
+        assert_equal(len(res), 1)
+
+        res = Annotation.search(user=user,
+                                query={'custom_field':'casesensitive'})
+        assert_equal(len(res), 0)
