@@ -1,13 +1,15 @@
 from . import TestCase
-from .helpers import MockUser, MockConsumer
+from .helpers import MockUser
 from nose.tools import *
 from mock import patch
 
-from flask import json, url_for
+from flask import json, g
+from six.moves import xrange
 
 from annotator import auth, es
 from annotator.annotation import Annotation
 
+
 class TestStore(TestCase):
     def setup(self):
         super(TestStore, self).setup()
@@ -52,25 +54,31 @@ class TestStore(TestCase):
         assert headers['Access-Control-Expose-Headers'] == 'Content-Length, Content-Type, Location', \
             "Did not send the right Access-Control-Expose-Headers header."
 
+    @patch('annotator.store.Annotation')
+    def test_pluggable_class(self, ann_mock):
+        g.annotation_class = ann_mock
+        response = self.cli.get('/api/annotations/testID', headers=self.headers)
+        ann_mock.return_value.fetch.assert_called_once()
+
     def test_index(self):
         response = self.cli.get('/api/annotations', headers=self.headers)
-        assert response.data == "[]", "response should be empty list"
+        assert response.data == b"[]", "response should be empty list"
 
     def test_create(self):
         payload = json.dumps({'name': 'Foo'})
+
         response = self.cli.post('/api/annotations',
                                  data=payload,
                                  content_type='application/json',
                                  headers=self.headers)
 
-        # import re
-        # See http://bit.ly/gxJBHo for details of this change.
-        # assert response.status_code == 303, "response should be 303 SEE OTHER"
-        # assert re.match(r"http://localhost/store/\d+", response.headers['Location']), "response should redirect to read_annotation url"
-
-        assert response.status_code == 200, "response should be 200 OK"
+        assert response.status_code == 201, "response should be 201 CREATED"
         data = json.loads(response.data)
         assert 'id' in data, "annotation id should be returned in response"
+        expected_location = '/api/annotations/{0}'.format(data['id'])
+        assert response.location.endswith(expected_location), (
+            "The response should have a Location header with the URL to read "
+            "the annotation that was created")
         assert data['user'] == self.user.id
         assert data['consumer'] == self.user.consumer.key
 
@@ -114,6 +122,30 @@ class TestStore(TestCase):
         assert ann['user'] == self.user.id, "annotation 'user' field should not be futzable by API"
         assert ann['consumer'] == self.user.consumer.key, "annotation 'consumer' field should not be used by API"
 
+    def test_create_should_not_update(self):
+        response = self.cli.post('/api/annotations',
+                                 data=json.dumps({'name': 'foo'}),
+                                 content_type='application/json',
+                                 headers=self.headers)
+        data = json.loads(response.data)
+        id_ = data['id']
+
+        # Try and update the annotation using the create API
+        response = self.cli.post('/api/annotations',
+                                 data=json.dumps({'name': 'bar', 'id': id_}),
+                                 content_type='application/json',
+                                 headers=self.headers)
+        data = json.loads(response.data)
+
+
+        assert id_ != data['id'], "create should always create a new annotation"
+
+        ann1 = self._get_annotation(id_)
+        ann2 = self._get_annotation(data['id'])
+
+        assert ann1['name'] == 'foo', "annotation name should be 'foo'"
+        assert ann2['name'] == 'bar', "annotation name should be 'bar'"
+
     @patch('annotator.store.json')
     @patch('annotator.store.Annotation')
     def test_create_refresh(self, ann_mock, json_mock):
@@ -266,13 +298,13 @@ class TestStore(TestCase):
         assert_equal(res['total'], 2)
         assert_equal(len(res['rows']), 2)
         assert_equal(res['rows'][0]['uri'], uri1)
-        assert_true(res['rows'][0]['id'] in [anno.id, anno2.id])
+        assert_true(res['rows'][0]['id'] in [anno['id'], anno2['id']])
 
     def test_search_limit(self):
         for i in xrange(250):
             self._create_annotation(refresh=False)
 
-        es.conn.refresh(timesleep=0.01)
+        es.conn.indices.refresh(es.index)
 
         # by default return 20
         res = self._get_search_results()
@@ -294,7 +326,7 @@ class TestStore(TestCase):
         for i in xrange(250):
             self._create_annotation(refresh=False)
 
-        es.conn.refresh(timesleep=0.01)
+        es.conn.indices.refresh(es.index)
 
         res = self._get_search_results()
         assert_equal(len(res['rows']), 20)
@@ -350,12 +382,30 @@ class TestStoreAuthz(TestCase):
         self.ctx.pop()
         super(TestStoreAuthz, self).teardown()
 
+    def test_index(self):
+        # Test unauthenticated
+        response = self.cli.get('/api/annotations')
+        results = json.loads(response.data)
+        assert results == [], "unauthenticated user should get an empty list"
+
+        # Test as bob (authorized to read)
+        response = self.cli.get('/api/annotations',
+                           headers=self.bob_headers)
+        results = json.loads(response.data)
+        assert results and results[0]['id'] == self.anno_id, "bob should see his own annotation"
+
+        # Test as charlie (unauthorized)
+        response = self.cli.get('/api/annotations',
+                           headers=self.charlie_headers)
+        results = json.loads(response.data)
+        assert results == []
+
     def test_read(self):
         response = self.cli.get('/api/annotations/123')
         assert response.status_code == 401, "response should be 401 NOT AUTHORIZED"
 
         response = self.cli.get('/api/annotations/123', headers=self.charlie_headers)
-        assert response.status_code == 401, "response should be 401 NOT AUTHORIZED"
+        assert response.status_code == 403, "response should be 403 FORBIDDEN"
 
         response = self.cli.get('/api/annotations/123', headers=self.alice_headers)
         assert response.status_code == 200, "response should be 200 OK"
@@ -372,7 +422,7 @@ class TestStoreAuthz(TestCase):
                                 data=payload,
                                 content_type='application/json',
                                 headers=self.bob_headers)
-        assert response.status_code == 401, "response should be 401 NOT AUTHORIZED"
+        assert response.status_code == 403, "response should be 403 FORBIDDEN"
 
         response = self.cli.put('/api/annotations/123',
                                 data=payload,
@@ -397,8 +447,8 @@ class TestStoreAuthz(TestCase):
                                 data=payload,
                                 content_type='application/json',
                                 headers=self.charlie_headers)
-        assert response.status_code == 401, "response should be 401 NOT AUTHORIZED"
-        assert 'permissions update' in response.data
+        assert response.status_code == 403, "response should be 403 FORBIDDEN"
+        assert b'permissions update' in response.data
 
         response = self.cli.put('/api/annotations/123',
                                 data=payload,
@@ -451,7 +501,7 @@ class TestStoreAuthz(TestCase):
         # Logged in as Bob: 1 result
         results = self._get_search_raw_results(headers=self.bob_headers)
         assert results['hits']['total'] == 1
-        assert results['hits']['hits'][0]['_source']['id'] == self.anno_id
+        assert results['hits']['hits'][0]['_id'] == self.anno_id
 
         # Logged in as Charlie: 0 results
         results = self._get_search_raw_results(headers=self.charlie_headers)
